@@ -20,6 +20,13 @@ extern "C" {
 #define WANT_CAPSTONE 1
 #define HAVE_LINUX_CAN_H 0
 
+#define R_BUILDSYSTEM "meson"
+#if 0 > 0
+#define R2_CSVERSION 4
+#else
+#define R2_CSVERSION 5
+#endif
+
 #define WITH_STATIC_THEMES False
 
 #define HAVE_GPERF 0
@@ -35,17 +42,17 @@ extern "C" {
 #define R2_LIBDIR "/usr/lib"
 #define R2_INCDIR "/usr/include/libr"
 #define R2_DATDIR "/usr/share"
-#define R2_WWWROOT "/usr/share/radare2/5.8.6/www"
-#define R2_PLUGINS "lib/radare2/5.8.6"
-#define R2_EXTRAS "lib/radare2-extras/5.8.6"
-#define R2_BINDINGS "lib/radare2-bindings/5.8.6"
+#define R2_WWWROOT "/usr/share/radare2/5.8.8/www"
+#define R2_PLUGINS "lib/radare2/5.8.8"
+#define R2_EXTRAS "lib/radare2-extras/5.8.8"
+#define R2_BINDINGS "lib/radare2-bindings/5.8.8"
 #define R2_DATDIR_R2 "share/radare2"
-#define R2_SDB "share/radare2/5.8.6"
-#define R2_ZIGNS "share/radare2/5.8.6/zigns"
-#define R2_THEMES "share/radare2/5.8.6/cons"
+#define R2_SDB "share/radare2/5.8.8"
+#define R2_ZIGNS "share/radare2/5.8.8/zigns"
+#define R2_THEMES "share/radare2/5.8.8/cons"
 #define R2_FORTUNES R_JOIN_3_PATHS ("/usr/share", "doc", "radare2")
-#define R2_FLAGS "share/radare2/5.8.6/flag"
-#define R2_HUD "share/radare2/5.8.6/hud"
+#define R2_FLAGS "share/radare2/5.8.8/flag"
+#define R2_HUD "share/radare2/5.8.8/hud"
 #else
 #if R2__WINDOWS__ || _MSC_VER
 #define R2_PREFIX "."
